@@ -19,10 +19,6 @@
 
 #include "config.h"
 
-#ifdef HAVE_POPPLER_FORM_FIELD_BUTTON_GET_BUTTON_TYPE
-#define HAVE_FORMS
-#endif
-
 #include <math.h>
 #include <string.h>
 #include <gtk/gtk.h>
@@ -68,15 +64,13 @@ typedef struct {
 typedef struct {
 	EvFileExporterFormat format;
 
-	gboolean landscape;
-	
 	/* Pages per sheet */
 	gint pages_per_sheet;
 	gint pages_printed;
 	gint pages_x;
 	gint pages_y;
-	gdouble page_width;
-	gdouble page_height;
+	gdouble paper_width;
+	gdouble paper_height;
 	
 #ifdef HAVE_CAIRO_PRINT
 	cairo_t *cr;
@@ -568,12 +562,6 @@ pdf_document_set_password (EvDocumentSecurity *document_security,
 	document->password = g_strdup (password);
 }
 
-static gboolean
-pdf_document_can_get_text (EvDocument *document)
-{
-	return TRUE;
-}
-
 static EvDocumentInfo *
 pdf_document_get_info (EvDocument *document)
 {
@@ -719,31 +707,6 @@ pdf_document_get_info (EvDocument *document)
 	return info;
 }
 
-static char *
-pdf_document_get_text (EvDocument *document, int page, EvRectangle *rect)
-{
-	PdfDocument *pdf_document = PDF_DOCUMENT (document);
-	PopplerPage *poppler_page;
-	PopplerRectangle r;
-	double height;
-	char *text;
-	
-	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	g_return_val_if_fail (poppler_page != NULL, NULL);
-
-	poppler_page_get_size (poppler_page, NULL, &height);
-	r.x1 = rect->x1;
-	r.y1 = height - rect->y2;
-	r.x2 = rect->x2;
-	r.y2 = height - rect->y1;
-
-	text = poppler_page_get_text (poppler_page, &r);
-
-	g_object_unref (poppler_page);
-
-	return text;
-}
-
 static void
 pdf_document_document_iface_init (EvDocumentIface *iface)
 {
@@ -755,8 +718,6 @@ pdf_document_document_iface_init (EvDocumentIface *iface)
 	iface->has_attachments = pdf_document_has_attachments;
 	iface->get_attachments = pdf_document_get_attachments;
 	iface->render = pdf_document_render;
-	iface->get_text = pdf_document_get_text;
-	iface->can_get_text = pdf_document_can_get_text;
 	iface->get_info = pdf_document_get_info;
 };
 
@@ -1095,8 +1056,11 @@ build_tree (PdfDocument      *pdf_document,
 				break;
 		}
 		
-		if (!link) {
+		if (!link || strlen (ev_link_get_title (link)) <= 0) {
 			poppler_action_free (action);
+			if (link)
+				g_object_unref (link);
+			
 			continue;
 		}
 
@@ -1583,7 +1547,6 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 	PdfDocument *pdf_document = PDF_DOCUMENT (exporter);
 	PdfPrintContext *ctx;
 	gdouble width, height;
-	gboolean landscape, change_orient = FALSE;
 #ifdef HAVE_CAIRO_PRINT
 	cairo_surface_t *surface = NULL;
 #endif
@@ -1595,8 +1558,8 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 	ctx->format = fc->format;
 	ctx->pages_per_sheet = fc->pages_per_sheet;
 
-	landscape = (fc->orientation == EV_FILE_EXPORTER_LANDSCAPE);
-	change_orient = landscape;
+	ctx->paper_width = fc->paper_width;
+	ctx->paper_height = fc->paper_height;
 	
 	switch (fc->pages_per_sheet) {
 	        default:
@@ -1605,7 +1568,6 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 			ctx->pages_y = 1;
 			break;
 	        case 2:
-			landscape = !landscape;
 			ctx->pages_x = 1;
 			ctx->pages_y = 2;
 			break;
@@ -1614,7 +1576,6 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 			ctx->pages_y = 2;
 			break;
 	        case 6:
-			landscape = !landscape;
 			ctx->pages_x = 2;
 			ctx->pages_y = 3;
 			break;
@@ -1628,36 +1589,12 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 			break;
 	}
 
-	ctx->landscape = landscape;
-
-	if (change_orient) {
-		width = fc->paper_height;
-		height = fc->paper_width;
-	} else {
-		width = fc->paper_width;
-		height = fc->paper_height;
-	}
-
-	if (landscape) {
-		gint tmp;
-
-		tmp = ctx->pages_x;
-		ctx->pages_x = ctx->pages_y;
-		ctx->pages_y = tmp;
-
-		ctx->page_width = height / ctx->pages_x;
-		ctx->page_height = width / ctx->pages_y;
-	} else {
-		ctx->page_width = width / ctx->pages_x;
-		ctx->page_height = height / ctx->pages_y;
-	}
-
 	ctx->pages_printed = 0;
 
 	switch (fc->format) {
 	        case EV_FILE_FORMAT_PS:
 #ifdef HAVE_CAIRO_PS
-			surface = cairo_ps_surface_create (fc->filename, width, height);
+			surface = cairo_ps_surface_create (fc->filename, fc->paper_width, fc->paper_height);
 #else
 			ctx->ps_file = poppler_ps_file_new (pdf_document->document,
 							    fc->filename, fc->first_page,
@@ -1668,7 +1605,7 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 			break;
 	        case EV_FILE_FORMAT_PDF:
 #ifdef HAVE_CAIRO_PDF
-			surface = cairo_pdf_surface_create (fc->filename, width, height);
+			surface = cairo_pdf_surface_create (fc->filename, fc->paper_width, fc->paper_height);
 #endif
 			break;
 	        default:
@@ -1677,16 +1614,6 @@ pdf_document_file_exporter_begin (EvFileExporter        *exporter,
 
 #ifdef HAVE_CAIRO_PRINT
 	ctx->cr = cairo_create (surface);
-	if (landscape) {
-		cairo_matrix_t matrix;
-		
-		cairo_translate (ctx->cr, width, 0);
-		cairo_matrix_init (&matrix,
-				   0,  1,
-				   -1,  0,
-				   0,  0);
-		cairo_transform (ctx->cr, &matrix);
-	}
 	cairo_surface_destroy (surface);
 #endif
 }
@@ -1699,8 +1626,12 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 	PdfPrintContext *ctx = pdf_document->print_ctx;
 	PopplerPage *poppler_page;
 #ifdef HAVE_CAIRO_PRINT
-	gdouble page_width, page_height;
-	gint    x, y;
+	gdouble  page_width, page_height;
+	gint     x, y;
+	gboolean rotate, landscape;
+	gdouble  width, height;
+	gdouble  pwidth, pheight;
+	gdouble  xscale, yscale;
 #endif
 
 	g_return_if_fail (pdf_document->print_ctx != NULL);
@@ -1711,21 +1642,83 @@ pdf_document_file_exporter_do_page (EvFileExporter  *exporter,
 	x = (ctx->pages_printed % ctx->pages_per_sheet) % ctx->pages_x;
 	y = (ctx->pages_printed % ctx->pages_per_sheet) / ctx->pages_x;
 	poppler_page_get_size (poppler_page, &page_width, &page_height);
-	
-	cairo_save (ctx->cr);
-	cairo_translate (ctx->cr,
-			 x * ctx->page_width,
-			 y * ctx->page_height);
-	if (ctx->landscape) {
-		cairo_scale (ctx->cr,
-			     ctx->page_height / page_height,
-			     ctx->page_height / page_height);
+
+	if (page_width > page_height && page_width > ctx->paper_width) {
+		rotate = TRUE;
 	} else {
-		cairo_scale (ctx->cr,
-			     ctx->page_width / page_width,
-			     ctx->page_height / page_height);
+		rotate = FALSE;
 	}
 
+	landscape = (ctx->paper_width > ctx->paper_height);
+
+	/* Use always portrait mode and rotate when necessary */
+	if (ctx->paper_width > ctx->paper_height) {
+		width = ctx->paper_height;
+		height = ctx->paper_width;
+		rotate = !rotate;
+
+		cairo_ps_surface_set_size (cairo_get_target (ctx->cr),
+					   width, height);
+	} else {
+		width = ctx->paper_width;
+		height = ctx->paper_height;
+	}
+
+	if (ctx->pages_per_sheet == 2 || ctx->pages_per_sheet == 6) {
+		rotate = !rotate;
+	}	
+
+	if (rotate) {
+		gint tmp1;
+		gdouble tmp2;
+
+		tmp1 = x;
+		x = y;
+		y = tmp1;
+
+		tmp2 = page_width;
+		page_width = page_height;
+		page_height = tmp2;
+
+	}
+
+	pwidth = width / ctx->pages_x;
+	pheight = height / ctx->pages_y;
+
+	if ((page_width > pwidth || page_height > pheight) ||
+	    (page_width < pwidth && page_height < pheight)) {
+		xscale = pwidth / page_width;
+		yscale = pheight / page_height;
+		
+		if (yscale < xscale) {
+			xscale = yscale;
+		} else {
+			yscale = xscale;
+		}
+		
+	} else {	
+		xscale = yscale = 1;
+	}
+
+	/* TODO: center */
+
+	cairo_save (ctx->cr);
+	if (rotate) {
+		cairo_matrix_t matrix;
+		
+		cairo_translate (ctx->cr, width, 0);
+		cairo_matrix_init (&matrix,
+				   0,  1,
+				   -1,  0,
+				   0,  0);
+		cairo_transform (ctx->cr, &matrix);
+	}
+	
+	cairo_translate (ctx->cr,
+			 x * (rotate ? pheight : pwidth),
+			 y * (rotate ? pwidth : pheight));
+	cairo_scale (ctx->cr, xscale, yscale);
+	
 #ifdef HAVE_POPPLER_PAGE_RENDER
 	poppler_page_render (poppler_page, ctx->cr);
 #endif
@@ -1792,6 +1785,7 @@ pdf_selection_render_selection (EvSelection      *selection,
 				cairo_surface_t **surface,
 				EvRectangle      *points,
 				EvRectangle      *old_points,
+				EvSelectionStyle  style,
 				GdkColor         *text,
 				GdkColor         *base)
 {
@@ -1807,7 +1801,6 @@ pdf_selection_render_selection (EvSelection      *selection,
 	width = (int) ((width_points * rc->scale) + 0.5);
 	height = (int) ((height_points * rc->scale) + 0.5);
 
-
 #ifdef HAVE_POPPLER_PAGE_RENDER
 	cairo_t *cr;
 
@@ -1827,7 +1820,7 @@ pdf_selection_render_selection (EvSelection      *selection,
 				       cr,
 				       (PopplerRectangle *)points,
 				       (PopplerRectangle *)old_points,
-				       POPPLER_SELECTION_NORMAL, /* SelectionStyle */
+				       (PopplerSelectionStyle)style,
 				       text,
 				       base);
 	cairo_destroy (cr);
@@ -1842,7 +1835,7 @@ pdf_selection_render_selection (EvSelection      *selection,
 						 rc->scale, rc->rotation, pixbuf,
 						 (PopplerRectangle *)points,
 						 (PopplerRectangle *)old_points,
-						 POPPLER_SELECTION_NORMAL, /* SelectionStyle */
+						 (PopplerSelectionStyle)style,
 						 text,
 						 base);
 	if (*surface)
@@ -1852,10 +1845,40 @@ pdf_selection_render_selection (EvSelection      *selection,
 #endif /* HAVE_POPPLER_PAGE_RENDER */
 }
 
+static gchar *
+pdf_selection_get_selected_text (EvSelection     *selection,
+				 EvRenderContext *rc,
+				 EvSelectionStyle style,
+				 EvRectangle     *points)
+{
+	PdfDocument *pdf_document = PDF_DOCUMENT (selection);
+	PopplerPage *poppler_page;
+	PopplerRectangle r;
+	double height;
+	char *retval;
+	
+	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
+	g_return_val_if_fail (poppler_page != NULL, NULL);
+
+	poppler_page_get_size (poppler_page, NULL, &height);
+	r.x1 = points->x1;
+	r.y1 = height - points->y2;
+	r.x2 = points->x2;
+	r.y2 = height - points->y1;
+
+	retval = poppler_page_get_text (poppler_page,
+					(PopplerSelectionStyle)style,
+					&r);
+
+	g_object_unref (poppler_page);
+
+	return retval;
+}
 
 static GdkRegion *
 pdf_selection_get_selection_region (EvSelection     *selection,
 				    EvRenderContext *rc,
+				    EvSelectionStyle style,
 				    EvRectangle     *points)
 {
 	PdfDocument *pdf_document;
@@ -1867,6 +1890,7 @@ pdf_selection_get_selection_region (EvSelection     *selection,
 
 	retval = poppler_page_get_selection_region ((PopplerPage *)rc->data,
 						    rc->scale,
+						    (PopplerSelectionStyle)style,
 						    (PopplerRectangle *) points);
 	return retval;
 }
@@ -1887,7 +1911,9 @@ pdf_selection_get_selection_map (EvSelection     *selection,
 	points.x1 = 0.0;
 	points.y1 = 0.0;
 	poppler_page_get_size (poppler_page, &(points.x2), &(points.y2));
-	retval = poppler_page_get_selection_region (poppler_page, 1.0, &points);
+	retval = poppler_page_get_selection_region (poppler_page, 1.0,
+						    POPPLER_SELECTION_GLYPH,
+						    &points);
 	g_object_unref (poppler_page);
 
 	return retval;
@@ -1897,6 +1923,7 @@ static void
 pdf_selection_iface_init (EvSelectionIface *iface)
 {
         iface->render_selection = pdf_selection_render_selection;
+	iface->get_selected_text = pdf_selection_get_selected_text;
         iface->get_selection_region = pdf_selection_get_selection_region;
         iface->get_selection_map = pdf_selection_get_selection_map;
 }
@@ -1952,7 +1979,6 @@ pdf_document_get_crop_box (EvDocument  *document,
 	rect->y2 = poppler_rect.y2;
 }
 
-#ifdef HAVE_FORMS
 static EvFormField *
 ev_form_field_from_poppler_field (PopplerFormField *poppler_field)
 {
@@ -1989,10 +2015,7 @@ ev_form_field_from_poppler_field (PopplerFormField *poppler_field)
 			field_text->do_scroll = poppler_form_field_text_do_scroll (poppler_field);
 			field_text->is_rich_text = poppler_form_field_text_is_rich_text (poppler_field);
 			field_text->is_password = poppler_form_field_text_is_password (poppler_field);
-			
-#ifdef HAVE_POPPLER_FORM_FIELD_TEXT_GET_MAX_LEN
 			field_text->max_len = poppler_form_field_text_get_max_len (poppler_field);
-#endif
 			field_text->text = poppler_form_field_text_get_text (poppler_field);
 
 		}
@@ -2321,4 +2344,4 @@ pdf_document_document_forms_iface_init (EvDocumentFormsIface *iface)
 	iface->form_field_choice_set_text = pdf_document_forms_form_field_choice_set_text;
 	iface->form_field_choice_get_text = pdf_document_forms_form_field_choice_get_text;
 }
-#endif /* HAVE_FORMS */
+
