@@ -4,11 +4,11 @@
 #include "ev-document-links.h"
 #include "ev-document-images.h"
 #include "ev-document-forms.h"
+#include "ev-file-exporter.h"
 #include "ev-document-factory.h"
 #include "ev-document-misc.h"
 #include "ev-file-helpers.h"
 #include "ev-document-fonts.h"
-#include "ev-selection.h"
 #include "ev-async-renderer.h"
 
 #include <glib/gstdio.h>
@@ -276,6 +276,7 @@ ev_job_render_new (EvDocument      *document,
 		   gint             width,
 		   gint             height,
 		   EvRectangle     *selection_points,
+		   EvSelectionStyle selection_style,
 		   GdkColor        *text,
 		   GdkColor        *base,
 		   gboolean 	    include_forms,
@@ -296,6 +297,7 @@ ev_job_render_new (EvDocument      *document,
 	job->rc = g_object_ref (rc);
 	job->target_width = width;
 	job->target_height = height;
+	job->selection_style = selection_style;
 	job->text = *text;
 	job->base = *base;
 	job->include_forms = include_forms;
@@ -371,10 +373,12 @@ ev_job_render_run (EvJobRender *job)
 						       &(job->selection),
 						       &(job->selection_points),
 						       NULL,
+						       job->selection_style,
 						       &(job->text), &(job->base));
 			job->selection_region =
 				ev_selection_get_selection_region (EV_SELECTION (EV_JOB (job)->document),
 								   job->rc,
+								   job->selection_style,
 								   &(job->selection_points));
 		}
 
@@ -562,7 +566,6 @@ ev_job_print_new (EvDocument    *document,
 		  const gchar   *format,
 		  gdouble        width,
 		  gdouble        height,
-		  EvFileExporterOrientation orientation,
 		  EvPrintRange  *ranges,
 		  gint           n_ranges,
 		  EvPrintPageSet page_set,
@@ -584,7 +587,6 @@ ev_job_print_new (EvDocument    *document,
 
 	job->width = width;
 	job->height = height;
-	job->orientation = orientation;
 
 	job->ranges = ranges;
 	job->n_ranges = n_ranges;
@@ -718,7 +720,6 @@ ev_job_print_run (EvJobPrint *job)
 	fc.last_page = MAX (first_page, last_page);
 	fc.paper_width = job->width;
 	fc.paper_height = job->height;
-	fc.orientation = job->orientation;
 	fc.duplex = FALSE;
 	fc.pages_per_sheet = MAX (1, job->pages_per_sheet);
 
